import { __decorate, __metadata } from 'tslib';
import { EventEmitter, Input, Component, ViewChild, ElementRef, KeyValueDiffers, Output, ViewContainerRef, ComponentFactoryResolver, Renderer2, TemplateRef, Injectable, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormControl, Validators, FormGroup, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { DaterangePickerComponent, DaterangepickerConfig, Daterangepicker } from 'ng2-daterangepicker';
import * as moment_ from 'moment';
import { TreeviewConfig, TreeviewComponent, TreeviewItem, TreeviewModule } from 'ngx-treeview';
import { TranslateModule } from '@ngx-translate/core';
import { MultiselectDropdownModule } from 'angular-2-dropdown-multiselect';
import { NgxSummernoteModule } from 'ngx-summernote';
import { HttpClientModule } from '@angular/common/http';

class FormField {
    constructor(values) {
        this.isRequired = false;
        Object.assign(this, values);
        this.control = new FormControl('');
        this.valueChange = new EventEmitter();
    }
    required() {
        this.isRequired = true;
        this.control.setValidators(Validators.required);
        return this;
    }
    setValidators(v) {
        this.control.setValidators(v);
        return this;
    }
    setValue(v) {
        this.control.setValue(v);
        this.valueChange.next(v);
        return this;
    }
    addValidator(v) {
        let validators = [];
        if (this.control.validator)
            validators.push(this.control.validator);
        validators.push(v);
        this.control.setValidators(validators);
        return this;
    }
}

let InputTextComponent = class InputTextComponent {
    clear() {
        this.request[this.field.id] = null;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], InputTextComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], InputTextComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], InputTextComponent.prototype, "request", void 0);
InputTextComponent = __decorate([
    Component({
        selector: 'tw-input-text',
        template: `
  <tw-template-field [field]='field'>
   <div [formGroup]='form' [ngClass]='field.more?.nullable ? "input-group" :""'>
  <input [formControlName]='field.id' name='{{field.id}}' type='{{field.type}}' placeholder='{{field.placeholder}}' class='form-control' [(ngModel)]="request[field.id]"
    [readonly]='field.more?.readonly' [email]='field.type==="email"' />
          <span class="input-group-btn" *ngIf='field.more?.nullable && field.type!=="hidden"'>
        <button class='btn btn-default' (click)='clear()'>&times;</button>
      </span>
</div>
</tw-template-field>
   `
    })
], InputTextComponent);

let InputFloatComponent = class InputFloatComponent {
    ngOnInit() {
        this.field.addValidator(Validators.pattern(/^[+-]?([0-9]*[.])?[0-9]+$/));
        this.sub = this.field.control.valueChanges.subscribe((value) => {
            this.request[this.field.id] = parseFloat(value);
        });
    }
    ngOnDestroy() {
        if (this.sub)
            this.sub.unsubscribe();
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], InputFloatComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], InputFloatComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], InputFloatComponent.prototype, "request", void 0);
InputFloatComponent = __decorate([
    Component({
        selector: 'tw-input-float',
        template: `
  <tw-template-field [field]='field'>
    <div [formGroup]='form'>
      <input placeholder='{{field.placeholder}}' [formControlName]='field.id' name='{{field.id}}' type='text' class='form-control' [readonly]='field.more?.readonly' />
    </div>
  </tw-template-field>`
    })
], InputFloatComponent);

let SelectComponent = class SelectComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], SelectComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], SelectComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectComponent.prototype, "request", void 0);
SelectComponent = __decorate([
    Component({
        selector: 'tw-select',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]">
  <option value="undefined" disabled selected>{{field.placeholder || ''}}</option>
  <option [ngValue]="field.firstValue.value " *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>
      <option [ngValue]="option[field.optionValue] " *ngFor="let option of field.options ">
        {{option[field.optionText]}}
      </option>
    </select>
</div>
  </tw-template-field>`
    })
], SelectComponent);

let SelectTComponent = class SelectTComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], SelectTComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], SelectTComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectTComponent.prototype, "request", void 0);
SelectTComponent = __decorate([
    Component({
        selector: 'tw-select-t',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]">
      <option [ngValue]="field.firstValue.value" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>
      <option [ngValue]="option[field.optionValue]" *ngFor="let option of field.options ">
        {{option[field.optionText] | translate}}
      </option>
    </select>
</div>
  </tw-template-field>`
    })
], SelectTComponent);

let EnumselectComponent = class EnumselectComponent {
    enumToArray(enumObj) {
        return Object.keys(enumObj).slice(Object.keys(enumObj).length / 2);
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], EnumselectComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], EnumselectComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], EnumselectComponent.prototype, "request", void 0);
EnumselectComponent = __decorate([
    Component({
        selector: 'tw-enumselect',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <select [attr.multiple]='field.type === "enumselect"?null:true' [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]">
      <option [ngValue]="field.firstValue.value" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>
      <option [ngValue]="option" *ngFor="let option of enumToArray(field.options)">
        {{option | translate}}
      </option>
    </select>
</div>
  </tw-template-field>`
    })
], EnumselectComponent);

const moment = moment_;
let DatetimePickerComponent = class DatetimePickerComponent {
    constructor() {
        this.daterange = {};
        this.minute = '';
        this.firstShow = true;
    }
    // differ: any;
    // constructor(private differs: KeyValueDiffers) {
    //   this.differ = differs.find({}).create();
    // }
    selectedDate(value, datepicker) {
        if (value.start) {
            if (!this.field.more.dateOnly)
                this.request[this.field.id] = value.start;
            else
                this.request[this.field.id] = value.start.hours(12);
            this.datepicker.nativeElement.value = value.start.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm');
        }
        this.field.control.setValue(this.datepicker.nativeElement.value);
    }
    showDaterangepicker(ev) {
        if (this.firstShow)
            this.selectedDate({ start: this.defaultValue });
        this.firstShow = false;
    }
    ngOnInit() {
        this.datepicker.nativeElement = this.datepicker.nativeElement;
        if (this.field.more) {
            if (this.field.more.defaultValue) {
                switch (this.field.more.defaultValue) {
                    case 'startOfDay':
                        this.defaultValue = moment().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue = moment().endOf('day');
                        break;
                    default:
                        this.defaultValue = this.field.more.defaultValue;
                        break;
                }
            }
            else {
                this.defaultValue = moment();
            }
        }
        else {
            this.field.more = {};
        }
        if (this.request[this.field.id]) {
            let v = moment(this.request[this.field.id]);
            this.defaultValue = v;
            this.field.control.setValue(v.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm'));
        }
        // if (this.field.more.dateOnly)
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01])$/))
        // else
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01]) (00|0[0-9]|1[0-9]|2[0-3]):([0-9]|[0-5][0-9])$/))
        this.options = {
            singleDatePicker: true,
            timePicker: !this.field.more.dateOnly,
            autoApply: true,
            timePicker24Hour: true,
            autoUpdateInput: false,
            startDate: this.defaultValue
        };
        if (this.field.more.options)
            for (let i in this.field.more.options)
                this.options[i] = this.field.more.options[i];
        this.sub = this.field.valueChange.subscribe((v) => {
            this.datepickerComponent.render();
            this.datepickerComponent.selected.next({ start: moment(v) });
            this.datepickerComponent.datePicker.setStartDate(moment(v));
            // this.selectedDate({ start: moment(v) })
        });
    }
    // valueChanged(date: Date): void {
    //   if (this.field.more.dateOnly && date)
    //     this.request[this.field.id] = date.toString()
    // }
    clear() {
        this.request[this.field.id] = null;
        this.datepicker.nativeElement.value = '';
        this.field.control.setValue('');
    }
    ngOnDestroy() {
        if (this.sub)
            this.sub.unsubscribe();
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], DatetimePickerComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], DatetimePickerComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DatetimePickerComponent.prototype, "request", void 0);
__decorate([
    ViewChild('datepicker'),
    __metadata("design:type", ElementRef)
], DatetimePickerComponent.prototype, "datepicker", void 0);
__decorate([
    ViewChild(DaterangePickerComponent),
    __metadata("design:type", DaterangePickerComponent)
], DatetimePickerComponent.prototype, "datepickerComponent", void 0);
DatetimePickerComponent = __decorate([
    Component({
        selector: 'tw-datetime-picker',
        //   template: `
        //   <div [formGroup]='form' class="input-group">
        //       <input [formControlName]='field.id' name='{{field.id}}' class='form-control' (valueChanged)='valueChanged($event)' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false" [(ngModel)]="request[field.id]" [date-only]="field.more.dateOnly ||false" [date-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"' [parse-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"'  readonly/>
        //       <span class="input-group-btn">
        //         <button class='btn btn-default' (click)='clear()'>&times;</button>
        //       </span>
        // </div>
        //   `
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form' class="input-group">
  <input placeholder='{{field.placeholder}}' #datepicker class='form-control' type="text" [formControlName]='field.id' name='{{field.id}}' id='{{field.id}}' daterangepicker [options]="options" (selected)="selectedDate($event, daterange)" (showDaterangepicker)="showDaterangepicker($event)"  readonly/>
  <span class="input-group-btn">
  <button class='btn btn-default' (click)='clear()'>&times;</button>
  </span>
  </div></tw-template-field>`
    })
], DatetimePickerComponent);

const moment$1 = moment_;
let DateRangePickerComponent = class DateRangePickerComponent {
    constructor(differs) {
        this.differs = differs;
        this.daterange = {};
        this.minute = '';
        this.firstShow = true;
        this.differ = differs.find({}).create();
    }
    selectedDate(value, datepicker) {
        if (value.start && value.end) {
            if (!this.field.more.dateOnly) {
                this.request[this.field.more.model1] = value.start;
                this.request[this.field.more.model2] = value.end;
            }
            else {
                this.request[this.field.more.model1] = value.start.hours(12);
                this.request[this.field.more.model2] = value.end.hours(12);
            }
            this.datepicker.nativeElement.value = value.start.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm') + ' - ' + value.end.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm');
        }
        this.field.control.setValue(this.datepicker.nativeElement.value);
    }
    showDaterangepicker(ev) {
        if (this.firstShow)
            this.selectedDate({ start: this.defaultValue1, end: this.defaultValue2 });
        this.firstShow = false;
    }
    ngOnInit() {
        this.datepicker.nativeElement = this.datepicker.nativeElement;
        if (this.field.more) {
            if (this.field.more.defaultValue1) {
                switch (this.field.more.defaultValue1) {
                    case 'startOfDay':
                        this.defaultValue1 = moment$1().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue1 = moment$1().endOf('day');
                        break;
                    default:
                        this.defaultValue1 = this.field.more.defaultValue1;
                        break;
                }
            }
            else {
                this.defaultValue1 = moment$1();
            }
            if (this.field.more.defaultValue2) {
                switch (this.field.more.defaultValue2) {
                    case 'startOfDay':
                        this.defaultValue2 = moment$1().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue2 = moment$1().endOf('day');
                        break;
                    default:
                        this.defaultValue2 = this.field.more.defaultValue2;
                        break;
                }
            }
            else {
                this.defaultValue2 = moment$1();
            }
        }
        else {
            this.field.more = {};
        }
        if (this.request[this.field.more.model1]) {
            let v = moment$1(this.request[this.field.more.model1]);
            this.defaultValue1 = v;
        }
        if (this.request[this.field.more.model2]) {
            let v = moment$1(this.request[this.field.more.model2]);
            this.defaultValue2 = v;
        }
        if (this.request[this.field.more.model1] && this.request[this.field.more.model2])
            this.field.control.setValue(this.defaultValue1.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm') + ' - ' + this.defaultValue2.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm'));
        // if (this.field.more.dateOnly)
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01])$/))
        // else
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01]) (00|0[0-9]|1[0-9]|2[0-3]):([0-9]|[0-5][0-9])$/))
        this.options = {
            timePicker: !this.field.more.dateOnly,
            timePicker24Hour: true,
            autoUpdateInput: false,
            autoApply: true,
            startDate: this.defaultValue1,
            endDate: this.defaultValue2
        };
        if (this.field.more.options)
            for (let i in this.field.more.options)
                this.options[i] = this.field.more.options[i];
    }
    // valueChanged(date: Date): void {
    //   if (this.field.more.dateOnly && date)
    //     this.request[this.field.id] = date.toString()
    // }
    clear() {
        this.request[this.field.more.model1] = null;
        this.request[this.field.more.model2] = null;
        this.datepicker.nativeElement.value = '';
        this.field.control.setValue('');
    }
    ngDoCheck() {
        var changes = this.differ.diff(this.request);
        if (changes) {
            let start = null;
            let end = null;
            changes.forEachChangedItem(r => {
                if (r.key === this.field.more.model1) {
                    start = r.currentValue;
                }
                else if (r.key === this.field.more.model2) {
                    end = r.currentValue;
                }
            });
            if (start || end) {
                this.selectedDate({ start: start || moment$1(this.request[this.field.more.model1]), end: end || moment$1(this.request[this.field.more.model2]) });
                this.firstShow = false;
            }
        }
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], DateRangePickerComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], DateRangePickerComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DateRangePickerComponent.prototype, "request", void 0);
__decorate([
    ViewChild('datepicker'),
    __metadata("design:type", ElementRef)
], DateRangePickerComponent.prototype, "datepicker", void 0);
DateRangePickerComponent = __decorate([
    Component({
        selector: 'tw-daterange-picker',
        //   template: `
        //   <div [formGroup]='form' class="input-group">
        //       <input [formControlName]='field.id' name='{{field.id}}' class='form-control' (valueChanged)='valueChanged($event)' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false" [(ngModel)]="request[field.id]" [date-only]="field.more.dateOnly ||false" [date-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"' [parse-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"'  readonly/>
        //       <span class="input-group-btn">
        //         <button class='btn btn-default' (click)='clear()'>&times;</button>
        //       </span>
        // </div>
        //   `
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form' class="input-group">
  <input #datepicker placeholder='{{field.placeholder}}' class='form-control' type="text" [formControlName]='field.id' name='{{field.id}}' daterangepicker [options]="options" (selected)="selectedDate($event, daterange)"  (showDaterangepicker)="showDaterangepicker($event)" readonly/>
  <span class="input-group-btn">
  <button class='btn btn-default' (click)='clear()'>&times;</button>
  </span>
  </div></tw-template-field>`
    }),
    __metadata("design:paramtypes", [KeyValueDiffers])
], DateRangePickerComponent);

let TimePickerComponent = 
// <div [formGroup]='form' class="input-group">
//   <input [formControlName]='field.id' name='{{field.id}}' class='form-control' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false"  (valueChanged)='valueChanged($event)'  [(ngModel)]="request[field.id]"  [time-only]="true" date-format='HH:mm' parse-format='HH:mm'  readonly/>
//       <span class="input-group-btn">
//       <button class='btn btn-default' (click)='clear()'>&times;</button>
//     </span>
// </div>
class TimePickerComponent {
    constructor() {
        this.defaultValue = '';
        this.hours = [];
        for (let i = 0; i <= 23; i++) {
            if (i < 10)
                this.hours.push('0' + i.toString());
            else
                this.hours.push(i.toString());
        }
        this.minutes = [];
        for (let i = 0; i <= 59; i++) {
            if (i < 10)
                this.minutes.push('0' + i.toString());
            else
                this.minutes.push(i.toString());
        }
    }
    // get model(): string {
    //   return this.request[this.field.id]
    // }
    // set model(m: string) {
    //   console.log(this.hour)
    // }
    onChange(hour) {
        if (hour && !this.minute)
            this.minute = '00';
        if (this.hour && this.minute)
            this.request[this.field.id] = this.hour + ':' + this.minute;
        else
            this.request[this.field.id] = null;
    }
    ngOnChanges(value) {
        let newV = value['request'].currentValue[this.field.id];
        if (newV) {
            let split = newV.split(':');
            this.hour = split[0] ? split[0] : '';
            this.minute = split[1] ? split[1] : '';
        }
    }
    ngOnInit() {
        if (this.field.more) {
            if (this.field.more.defaultValue) ;
        }
        else {
            this.field.more = {};
        }
        // this.field.addValidator(Validators.pattern(/^(00|0[0-9]|1[0-9]|2[0-3]):(0[0-9]|[0-5][0-9])(:([0-9]|[0-5][0-9]))?$/))
    }
    // valueChanged(date: Date): void {
    //   // console.log('time changed')
    //   if (date)
    //     this.request[this.field.id] = date.toString()
    // }
    clear() {
        this.hour = '';
        this.minute = '';
        this.request[this.field.id] = null;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TimePickerComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TimePickerComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TimePickerComponent.prototype, "request", void 0);
TimePickerComponent = __decorate([
    Component({
        selector: 'tw-time-picker',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form' style='display:none'>
  <input type='hidden'  [formControlName]='field.id' [(ngModel)]="this.request[this.field.id]" />
  </div>
  <div class='clear' >
   <select [(ngModel)]="hour" (ngModelChange)='onChange(true)' class='form-control pull-left' style='width:100px;'>
        <option [ngValue]=""></option>
        <option *ngFor="let h of hours" [ngValue]="h">{{h}}</option>
    </select>
    <span class='pull-left' style='margin:5px'>hh</span>
     <select [(ngModel)]="minute" (ngModelChange)='onChange()'  class='form-control pull-left' style='width:100px;'>
        <option [ngValue]=""></option>
        <option *ngFor="let m of minutes" [ngValue]="m">{{m}}</option>
    </select>
    <span class='pull-left' style='margin:5px'>mm</span>
      <button class='btn btn-default pull-left'  (click)='clear()'>&times;</button>
  </div>
    </tw-template-field>`
    })
    // <div [formGroup]='form' class="input-group">
    //   <input [formControlName]='field.id' name='{{field.id}}' class='form-control' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false"  (valueChanged)='valueChanged($event)'  [(ngModel)]="request[field.id]"  [time-only]="true" date-format='HH:mm' parse-format='HH:mm'  readonly/>
    //       <span class="input-group-btn">
    //       <button class='btn btn-default' (click)='clear()'>&times;</button>
    //     </span>
    // </div>
    ,
    __metadata("design:paramtypes", [])
], TimePickerComponent);

let SwitchComponent = class SwitchComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], SwitchComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], SwitchComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SwitchComponent.prototype, "request", void 0);
SwitchComponent = __decorate([
    Component({
        selector: 'tw-switch',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <label class="switch switch-icon switch-pill switch-primary-outline-alt">
        <input type="checkbox" [formControlName]='field.id' class="switch-input"  [(ngModel)]="request[field.id]" >
        <span class="switch-label" data-on="yes" data-off="no"></span>
        <span class="switch-handle"></span>
    </label>
</div>
  </tw-template-field>`
    })
], SwitchComponent);

let GroupSelectComponent = class GroupSelectComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], GroupSelectComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], GroupSelectComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], GroupSelectComponent.prototype, "request", void 0);
GroupSelectComponent = __decorate([
    Component({
        selector: 'tw-groupselect',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]" >
        <option [ngValue]="field.firstValue.value " *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>
        <optgroup label="{{group.label}}" *ngFor="let group of field.options">
        <option [ngValue]="option[field.optionValue]" *ngFor="let option of group.values">
          {{option[field.optionText]}}
        </option>
      </optgroup>
    </select>
</div>
  </tw-template-field>`
    })
], GroupSelectComponent);

let MultiSelectComponent = class MultiSelectComponent {
    ngOnInit() {
        if (!this.field.more)
            this.field.more = {};
    }
    get allSelected() {
        return this.testSelected();
    }
    selectAll() {
        if (this.testSelected())
            this.request[this.field.id] = [];
        else {
            this.request[this.field.id] = [];
            this.field.options.forEach((option) => {
                this.request[this.field.id].push(option[this.field.optionValue]);
            });
        }
    }
    testSelected() {
        if (!this.request[this.field.id] || !this.field.options)
            return false;
        if (this.request[this.field.id].length === this.field.options.length)
            return true;
        if (this.request[this.field.id].length === this.field.options.size)
            return true;
        return false;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], MultiSelectComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], MultiSelectComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], MultiSelectComponent.prototype, "request", void 0);
MultiSelectComponent = __decorate([
    Component({
        selector: 'tw-multiselect',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
   <button style='margin-top: -30px; margin-bottom: 5px;' [ngClass]="allSelected ? 'btn-primary' : 'btn-default'" class='btn-sm btn pull-right' (click)='selectAll()' > {{field.more.allText || 'All'}}</button>
<select multiple [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]">
    <option [ngValue]="option[field.optionValue]" *ngFor="let option of field.options">
      {{option[field.optionText]}}
    </option>
  </select>
  </div>
  </tw-template-field>`
    })
], MultiSelectComponent);

let MultiSelectCBComponent = class MultiSelectCBComponent {
    ngOnInit() {
        if (!this.field.more)
            this.field.more = {};
    }
    get allSelected() {
        return this.testSelected();
    }
    get options() {
        if (this.field.options)
            return this.field.options.map((opt) => {
                opt.id = opt[this.field.optionValue];
                opt.name = opt[this.field.optionText];
                return opt;
            });
        return [];
    }
    selectAll() {
        if (this.testSelected())
            this.request[this.field.id] = [];
        else {
            this.request[this.field.id] = [];
            this.field.options.forEach((option) => {
                this.request[this.field.id].push(option[this.field.optionValue]);
            });
        }
    }
    testSelected() {
        if (!this.request[this.field.id] || !this.field.options)
            return false;
        if (this.request[this.field.id].length === this.field.options.length)
            return true;
        if (this.request[this.field.id].length === this.field.options.size)
            return true;
        return false;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], MultiSelectCBComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], MultiSelectCBComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], MultiSelectCBComponent.prototype, "request", void 0);
MultiSelectCBComponent = __decorate([
    Component({
        selector: 'tw-multiselectcb',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
    <button style='margin-top: -30px; margin-bottom: 5px;' [ngClass]="allSelected ? 'btn-primary' : 'btn-default'" class='btn-sm btn pull-right' (click)='selectAll()' > {{field.more.allText || 'All'}}</button>
    <ss-multiselect-dropdown [formControlName]='field.id' [options]="options" [(ngModel)]="request[field.id]" ></ss-multiselect-dropdown>
  </div>
  </tw-template-field>`
    })
], MultiSelectCBComponent);

let TextAreaComponent = class TextAreaComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TextAreaComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TextAreaComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TextAreaComponent.prototype, "request", void 0);
TextAreaComponent = __decorate([
    Component({
        selector: 'tw-textarea',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
   <textarea placeholder='{{field.placeholder}}' [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]"></textarea>
</div>
  </tw-template-field>`
    })
], TextAreaComponent);

let DatePickerComponent = class DatePickerComponent {
    constructor() {
        this.defaultValue = '';
        this.closeOnSelect = false;
    }
    // private sub: Subscription
    ngOnInit() {
        if (!this.field.more)
            this.field.more = {};
        this.field.more.dateOnly = true;
        if (!this.field.more.dateFormat)
            this.field.more.dateFormat = 'YYYY-MM-DD';
    }
    ngOnDestroy() {
        // if (this.sub)
        //   this.sub.unsubscribe()
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], DatePickerComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], DatePickerComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DatePickerComponent.prototype, "request", void 0);
DatePickerComponent = __decorate([
    Component({
        selector: 'tw-date-picker',
        template: `<tw-datetime-picker [form]='form' [field]='field'  [request]='request'></tw-datetime-picker>`
    })
], DatePickerComponent);

let DaysOfWeekComponent = class DaysOfWeekComponent {
    ngOnInit() {
        this.field.options = DaysOfWeekValues;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], DaysOfWeekComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], DaysOfWeekComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], DaysOfWeekComponent.prototype, "request", void 0);
DaysOfWeekComponent = __decorate([
    Component({
        selector: 'tw-dof',
        template: `<tw-enumselect [form]='form' [field]='field' [request]='request'></tw-enumselect>`
    })
], DaysOfWeekComponent);
var DaysOfWeekValues;
(function (DaysOfWeekValues) {
    DaysOfWeekValues[DaysOfWeekValues["Sunday"] = 0] = "Sunday";
    DaysOfWeekValues[DaysOfWeekValues["Monday"] = 1] = "Monday";
    DaysOfWeekValues[DaysOfWeekValues["Tuesday"] = 2] = "Tuesday";
    DaysOfWeekValues[DaysOfWeekValues["Wednesday"] = 3] = "Wednesday";
    DaysOfWeekValues[DaysOfWeekValues["Thursday"] = 4] = "Thursday";
    DaysOfWeekValues[DaysOfWeekValues["Friday"] = 5] = "Friday";
    DaysOfWeekValues[DaysOfWeekValues["Saturday"] = 6] = "Saturday";
})(DaysOfWeekValues || (DaysOfWeekValues = {}));

let TreeViewTxtComponent = class TreeViewTxtComponent {
    constructor() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    checkItem(item) {
        let selection = item.getSelection();
        let values = [];
        if (item.children) {
            if (selection.uncheckedItems.length === 0)
                values.push(item.value);
            for (const child of item.children)
                values = values.concat(this.checkItem(child));
        }
        return values;
    }
    updateModel(values) {
        /*Select Parent*/
        if (this.treeviewComponent) {
            for (const item of this.treeviewComponent.items) {
                values = values.concat(this.checkItem(item));
            }
        }
        if (values)
            this.field.setValue(values.join(','));
        else
            this.field.setValue('');
    }
    ngOnInit() {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TreeViewTxtComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TreeViewTxtComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TreeViewTxtComponent.prototype, "request", void 0);
__decorate([
    ViewChild(TreeviewComponent),
    __metadata("design:type", TreeviewComponent)
], TreeViewTxtComponent.prototype, "treeviewComponent", void 0);
TreeViewTxtComponent = __decorate([
    Component({
        selector: 'tw-treeview-txt',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
   <ngx-treeview #treeview *ngIf='field.type==="treeview-txt"' id="treeview_{{field.id}}" [config]="selectConfig" [items]="field.options" (selectedChange)="updateModel($event)"></ngx-treeview>
   <ngx-dropdown-treeview *ngIf='field.type==="dropdown-treeview-txt"' id="treeview_{{field.id}}" [config]="selectConfig" [items]="field.options" (selectedChange)="updateModel($event)"></ngx-dropdown-treeview>
    <input [formControlName]='field.id' name='{{field.id}}' type='hidden' class='form-control' [(ngModel)]="request[field.id]"/>
</div>
  </tw-template-field>`
    })
], TreeViewTxtComponent);

let TreeViewRadioComponent = class TreeViewRadioComponent {
    constructor() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    toggleCollapseExpand() {
        this.allItem.collapsed = !this.allItem.collapsed;
        if (this.field.options.items) {
            this.field.options.items.forEach((item) => item.setCollapsedRecursive(this.allItem.collapsed));
        }
    }
    updateModel(values) {
        if (values)
            this.field.setValue(values.join(','));
        else
            this.field.setValue('');
    }
    ngOnInit() {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TreeViewRadioComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TreeViewRadioComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TreeViewRadioComponent.prototype, "request", void 0);
TreeViewRadioComponent = __decorate([
    Component({
        selector: 'tw-treeview-radio',
        template: `<tw-template-field [field]='field'>
<ng-template #tpl let-item="item"
    let-toggleCollapseExpand="toggleCollapseExpand"
    let-onCheckedChange="onCheckedChange">
    <div class="form-check" [formGroup]='form'>
        <i *ngIf="item.children" (click)="toggleCollapseExpand()" aria-hidden="true"
            class="fa" [class.fa-caret-right]="item.collapsed" [class.fa-caret-down]="!item.collapsed"></i>
        <label class="form-check-label">
            <input type="radio"  class="form-check-input" [(ngModel)]="request[field.id]" [formControlName]='field.id'  name='{{field.id}}' [value]='item.value' />
            {{item.text}}
        </label>
    </div>
</ng-template>
<div class="treeview-header">

</div>
<div class="treeview-container" [style.max-height.px]="selectConfig.maxHeight" >
    <div *ngFor="let item of field.options">
        <tw-treeview-item [item]="item" [template]=" tpl" >
        </tw-treeview-item>
    </div>
</div>
</tw-template-field>`,
        styles: [`
.row-margin {
    margin-top: .3rem;
}
.label-item-all {
}
.label-collapse-expand {
    margin: 0;
    padding: 0 .3rem;
    cursor: pointer;
}
.divider {
    height: 1px;
    margin: 0.5rem 0;
    overflow: hidden;
    background: #000;
}
.treeview-container {
    overflow-x: hidden;
    overflow-y: auto;
    padding-right: 18px;
}
.treeview-text {
    padding: .3rem 0;
    white-space: nowrap;
}
`]
    })
], TreeViewRadioComponent);

let TreeViewComponent = class TreeViewComponent {
    constructor() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    checkItem(item) {
        let selection = item.getSelection();
        let values = [];
        if (item.children) {
            if (selection.uncheckedItems.length === 0)
                values.push(item.value);
            for (const child of item.children)
                values = values.concat(this.checkItem(child));
        }
        return values;
    }
    updateModel(values) {
        /*Select Parent*/
        if (this.treeviewComponent) {
            for (const item of this.treeviewComponent.items) {
                values = values.concat(this.checkItem(item));
            }
        }
        if (values)
            this.field.setValue(values);
        else
            this.field.setValue([]);
    }
    ngOnInit() {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TreeViewComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TreeViewComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], TreeViewComponent.prototype, "request", void 0);
__decorate([
    ViewChild(TreeviewComponent),
    __metadata("design:type", TreeviewComponent)
], TreeViewComponent.prototype, "treeviewComponent", void 0);
TreeViewComponent = __decorate([
    Component({
        selector: 'tw-treeview',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
   <ngx-treeview *ngIf='field.type==="treeview"' [config]="selectConfig" [items]="field.options" (selectedChange)="updateModel($event)"></ngx-treeview>
   <ngx-dropdown-treeview *ngIf='field.type==="dropdown-treeview"'  [config]="selectConfig" [items]="field.options" (selectedChange)="updateModel($event)"></ngx-dropdown-treeview>
  <input [formControlName]='field.id' name='{{field.id}}' type='hidden' class='form-control' [(ngModel)]="request[field.id]"/>
</div>
  </tw-template-field>`
    })
], TreeViewComponent);

let SelectArrayComponent = class SelectArrayComponent {
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], SelectArrayComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], SelectArrayComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SelectArrayComponent.prototype, "request", void 0);
SelectArrayComponent = __decorate([
    Component({
        selector: 'tw-select-array',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]="request[field.id]">
      <option [ngValue]="null" selected disabled  >{{field.firstValue | translate}}</option>
      <option [ngValue]="option" *ngFor="let option of field.options ">
        {{option}}
      </option>
    </select>
</div>
  </tw-template-field>`
    })
], SelectArrayComponent);

let HtmlComponent = class HtmlComponent {
    constructor() {
        this.config = {
            height: '250px',
            toolbar: [
                ['misc', ['undo', 'redo', 'codeBlock']],
                ['font', ['bold', 'italic', 'underline', 'strikethrough', 'superscript', 'subscript', 'clear']],
                ['fontsize', ['fontname', 'fontsize', 'color']],
                ['para', ['style0', 'ul', 'ol', 'paragraph', 'height']],
                ['insert', ['table', 'picture', 'link', 'video', 'hr']],
            ]
        };
        // public onChange({ editor }: ChangeEvent) {
        //   this.request[this.field.id] = editor.getData();
        // }
    }
    ngOnInit() {
        if (!this.request[this.field.id])
            this.request[this.field.id] = '';
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], HtmlComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormField)
], HtmlComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], HtmlComponent.prototype, "request", void 0);
HtmlComponent = __decorate([
    Component({
        selector: 'tw-html',
        template: `<tw-template-field [field]='field'>
  <div [formGroup]='form'>
  <div [ngxSummernote]="field?.more?.config || config" [ngxSummernoteView]="request[field.id]"  [formControlName]='field.id' [(ngModel)]="request[field.id]"></div>
  </div>
  </tw-template-field>`
    })
], HtmlComponent);

let ReactiveFormComponent = class ReactiveFormComponent {
    constructor(componentFactoryResolver, renderer) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.renderer = renderer;
        this.fields = [];
        this.events = []; // use later to display form changes
        this.afterInit = new EventEmitter(true);
        this.afterViewInit = new EventEmitter(true);
    }
    ngOnChanges(values) {
        if (values.fields && !values.fields.firstChange)
            this.refreshComponentsFields();
        if (values.request && !values.request.firstChange)
            this.components.forEach((c) => c.request = values.request.currentValue);
        if (values.form && !values.form.firstChange)
            this.components.forEach((c) => c.form = values.form.currentValue);
    }
    ngOnInit() {
        this.refreshComponentsFields();
        this.afterInit.next(true);
    }
    ngAfterViewInit() {
        this.afterViewInit.next(true);
    }
    refreshComponentsFields() {
        this.vc.clear();
        this.components = [];
        this.fields.forEach((field) => {
            this.form.addControl(field.id, field.control);
            this.components.push(this.loadComponentFields(field));
        });
    }
    loadComponentFields(f) {
        const componentFactory = this.componentFactoryResolver.resolveComponentFactory(this.getComponent(f));
        const componentRef = this.vc.createComponent(componentFactory);
        componentRef.instance.field = f;
        componentRef.instance.form = this.form;
        componentRef.instance.request = this.request;
        this.renderer.addClass(componentRef.location.nativeElement, 'form-group');
        this.renderer.addClass(componentRef.location.nativeElement, 'col-' + (f.col || 'sm-6'));
        if (f.class)
            this.renderer.addClass(componentRef.location.nativeElement, f.class);
        return componentRef.instance;
    }
    getComponent(f) {
        let comp = null;
        switch (f.type) {
            case 'float':
                comp = InputFloatComponent;
                break;
            case 'select':
                comp = SelectComponent;
                break;
            case 'selectT':
                comp = SelectTComponent;
                break;
            case 'arrayselect':
                comp = SelectArrayComponent;
                break;
            case 'enumselect':
                comp = EnumselectComponent;
                break;
            case 'datetime':
                comp = DatetimePickerComponent;
                break;
            case 'daterange':
                comp = DateRangePickerComponent;
                break;
            case 'time':
                comp = TimePickerComponent;
                break;
            case 'boolean':
                comp = SwitchComponent;
                break;
            case 'groupselect':
                comp = GroupSelectComponent;
                break;
            case 'multiselect':
                comp = MultiSelectComponent;
                break;
            case 'multiselectcb':
                comp = MultiSelectCBComponent;
                break;
            case 'textarea':
                comp = TextAreaComponent;
                break;
            case 'date':
                comp = DatePickerComponent;
                break;
            case 'daysOfWeek':
                comp = DaysOfWeekComponent;
                break;
            case 'treeview-txt':
                comp = TreeViewTxtComponent;
                break;
            case 'treeview-radio':
                comp = TreeViewRadioComponent;
                break;
            case 'treeview':
                comp = TreeViewComponent;
                break;
            case 'html':
                comp = HtmlComponent;
                break;
            case 'text':
            default:
                comp = InputTextComponent;
                break;
        }
        return comp;
    }
};
__decorate([
    Input(),
    __metadata("design:type", Array)
], ReactiveFormComponent.prototype, "fields", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], ReactiveFormComponent.prototype, "form", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], ReactiveFormComponent.prototype, "request", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], ReactiveFormComponent.prototype, "afterInit", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], ReactiveFormComponent.prototype, "afterViewInit", void 0);
__decorate([
    ViewChild('vc', { read: ViewContainerRef }),
    __metadata("design:type", Object)
], ReactiveFormComponent.prototype, "vc", void 0);
ReactiveFormComponent = __decorate([
    Component({
        selector: 'tw-reactive-form',
        template: `
  <form [formGroup]="form" >
  <div class="row" >
  <ng-container #vc></ng-container>
  </div>
  </form>
    `
    }),
    __metadata("design:paramtypes", [ComponentFactoryResolver, Renderer2])
], ReactiveFormComponent);

// import { TreeviewItemTemplateContext } from 'ng2-dropdown-treeview/src/treeview-item-template-context';
let TreeViewItemComponent = class TreeViewItemComponent {
    // import { TreeviewItemTemplateContext } from 'ng2-dropdown-treeview/src/treeview-item-template-context';
    constructor() {
        this.toggleCollapseExpand = () => {
            this.item.collapsed = !this.item.collapsed;
        };
    }
};
__decorate([
    Input(),
    __metadata("design:type", TemplateRef)
], TreeViewItemComponent.prototype, "template", void 0);
__decorate([
    Input(),
    __metadata("design:type", TreeviewItem)
], TreeViewItemComponent.prototype, "item", void 0);
TreeViewItemComponent = __decorate([
    Component({
        selector: 'tw-treeview-item',
        template: `
<div class="treeview-item">
    <ng-template [ngTemplateOutlet]="template"
        [ngTemplateOutletContext]="{item: item, toggleCollapseExpand: toggleCollapseExpand}">
    </ng-template>
    <div *ngIf="!item.collapsed">
        <tw-treeview-item *ngFor="let child of item.children" [item]="child" [template]="template">
        </tw-treeview-item>
    </div>
</div>
    `,
        styles: [`
:host {
    display: block;
}
:host /deep/ .fa {
    margin-right: .2rem;
    width: .5rem;
    cursor: pointer;
}
.treeview-item {
    white-space: nowrap;
}
.treeview-item .treeview-item {
    margin-left: 2rem;
}
    `]
    })
], TreeViewItemComponent);

let BusyService = class BusyService {
    constructor() {
        this.global = false;
    }
    start() {
        this.global = true;
    }
    stop() {
        this.global = false;
    }
};
BusyService = __decorate([
    Injectable()
], BusyService);

let BusyButtonComponent = class BusyButtonComponent {
    constructor(busy) {
        this.busy = busy;
        this.onClick = new EventEmitter(true);
    }
    send() {
        this.onClick.next(true);
    }
};
__decorate([
    Input(),
    __metadata("design:type", String)
], BusyButtonComponent.prototype, "cssClass", void 0);
__decorate([
    Input(),
    __metadata("design:type", Boolean)
], BusyButtonComponent.prototype, "disabled", void 0);
__decorate([
    Input(),
    __metadata("design:type", String)
], BusyButtonComponent.prototype, "icon", void 0);
__decorate([
    Output(),
    __metadata("design:type", EventEmitter)
], BusyButtonComponent.prototype, "onClick", void 0);
BusyButtonComponent = __decorate([
    Component({
        selector: 'tw-busy-button',
        template: '<button (click)=\'send()\' [class]="cssClass" [disabled]="disabled || busy.global"><span style=\'margin- right:5px\'  *ngIf=\'busy.global\'><img src="assets/img/ajax-loader.gif"/></span><i class=\'fa fa-{{icon }}\' style=\'margin- right:5px\'  *ngIf=\'icon && !busy.global\'></i><ng-content></ng-content></button>'
    }),
    __metadata("design:paramtypes", [BusyService])
], BusyButtonComponent);

let AngularFormsUtilsConfigs = class AngularFormsUtilsConfigs {
    constructor(daterangepickerConfig) {
        this.daterangepickerConfig = daterangepickerConfig;
    }
};
AngularFormsUtilsConfigs = __decorate([
    Injectable(),
    __metadata("design:paramtypes", [DaterangepickerConfig])
], AngularFormsUtilsConfigs);

let TemplateFieldComponent = class TemplateFieldComponent {
    get errors() {
        const errors = [];
        //
        if (this.field.control.errors != null) {
            Object.keys(this.field.control.errors).forEach(keyError => {
                errors.push(`form.errors.${keyError}`);
            });
        }
        return errors;
    }
};
__decorate([
    Input(),
    __metadata("design:type", FormField)
], TemplateFieldComponent.prototype, "field", void 0);
__decorate([
    Input(),
    __metadata("design:type", FormGroup)
], TemplateFieldComponent.prototype, "form", void 0);
TemplateFieldComponent = __decorate([
    Component({
        selector: 'tw-template-field',
        template: `
  <label [ngClass]='{"text-danger form-errors":(field.control.invalid && (field.control.dirty || field.control.touched))}' for="{{field.id}}"><span *ngIf='field.isRequired'>* </span>{{ field.label | translate }} <small *ngIf='field.info' > {{ field.info | translate }}</small></label>
    <ng-content ></ng-content>
     <ul *ngIf="field.control.invalid && (field.control.dirty || field.control.touched)" class="text-danger form-errors">
        <li *ngFor="let error of errors">{{error | translate}}</li>
    </ul>
      `
    })
], TemplateFieldComponent);

var AngularFormsUtilsModule_1;
// NguiDatetime.parseDate = (str: any): any => {
//   let m: any = moment(str)
//   if (!m.isValid()) { //Time
//     // let tmp = str.split(':');
//     m = moment()
//     // m.set('hour', parseInt(tmp[0] || '0', 10))
//     // m.set('minute', parseInt(tmp[1] || '0', 10))
//     // m.set('second', parseInt(tmp[2] || '0', 10))
//   }
//   return m.toDate()
// }
let decExp = [
    TemplateFieldComponent,
    ReactiveFormComponent,
    DatetimePickerComponent,
    MultiSelectCBComponent,
    SelectComponent,
    EnumselectComponent,
    InputTextComponent,
    InputFloatComponent,
    TimePickerComponent,
    SwitchComponent,
    GroupSelectComponent,
    DatePickerComponent,
    DaysOfWeekComponent,
    MultiSelectComponent,
    TextAreaComponent,
    BusyButtonComponent,
    TreeViewComponent,
    TreeViewTxtComponent,
    TreeViewRadioComponent,
    TreeViewItemComponent,
    SelectTComponent,
    DateRangePickerComponent,
    SelectArrayComponent,
    HtmlComponent
];
let AngularFormsUtilsModule = AngularFormsUtilsModule_1 = class AngularFormsUtilsModule {
    static forRoot() {
        return { ngModule: AngularFormsUtilsModule_1, providers: [BusyService] };
    }
};
AngularFormsUtilsModule = AngularFormsUtilsModule_1 = __decorate([
    NgModule({
        imports: [
            ReactiveFormsModule,
            CommonModule,
            TranslateModule,
            MultiselectDropdownModule,
            // NguiDatetimePickerModule,
            FormsModule,
            TreeviewModule,
            Daterangepicker,
            // FroalaEditorModule.forRoot(),
            // FroalaViewModule.forRoot(),
            NgxSummernoteModule,
            HttpClientModule
        ],
        declarations: decExp,
        exports: decExp,
        entryComponents: decExp,
        providers: [AngularFormsUtilsConfigs]
    })
], AngularFormsUtilsModule);

/**
 * Generated bundle index. Do not edit.
 */

export { AngularFormsUtilsConfigs, AngularFormsUtilsModule, BusyService, FormField, TemplateFieldComponent as ɵa, ReactiveFormComponent as ɵb, DatetimePickerComponent as ɵc, MultiSelectCBComponent as ɵd, SelectComponent as ɵe, EnumselectComponent as ɵf, InputTextComponent as ɵg, InputFloatComponent as ɵh, TimePickerComponent as ɵi, SwitchComponent as ɵj, GroupSelectComponent as ɵk, DatePickerComponent as ɵl, DaysOfWeekComponent as ɵm, MultiSelectComponent as ɵn, TextAreaComponent as ɵo, BusyButtonComponent as ɵp, TreeViewComponent as ɵq, TreeViewTxtComponent as ɵr, TreeViewRadioComponent as ɵs, TreeViewItemComponent as ɵt, SelectTComponent as ɵu, DateRangePickerComponent as ɵv, SelectArrayComponent as ɵw, HtmlComponent as ɵx };
//# sourceMappingURL=angular-forms-utils.js.map
