import { __decorate, __metadata, __values } from 'tslib';
import { EventEmitter, Input, Component, ViewChild, ElementRef, KeyValueDiffers, Output, ViewContainerRef, ComponentFactoryResolver, Renderer2, TemplateRef, Injectable, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Validators, FormControl, FormGroup, ReactiveFormsModule, FormsModule } from '@angular/forms';
import { DaterangePickerComponent, DaterangepickerConfig, Daterangepicker } from 'ng2-daterangepicker';
import * as moment_ from 'moment';
import { TreeviewComponent, TreeviewConfig, TreeviewItem, TreeviewModule } from 'ngx-treeview';
import { TranslateModule } from '@ngx-translate/core';
import { MultiselectDropdownModule } from 'angular-2-dropdown-multiselect';
import { NgxSummernoteModule } from 'ngx-summernote';
import { HttpClientModule } from '@angular/common/http';

var FormField = /** @class */ (function () {
    function FormField(values) {
        this.isRequired = false;
        Object.assign(this, values);
        this.control = new FormControl('');
        this.valueChange = new EventEmitter();
    }
    FormField.prototype.required = function () {
        this.isRequired = true;
        this.control.setValidators(Validators.required);
        return this;
    };
    FormField.prototype.setValidators = function (v) {
        this.control.setValidators(v);
        return this;
    };
    FormField.prototype.setValue = function (v) {
        this.control.setValue(v);
        this.valueChange.next(v);
        return this;
    };
    FormField.prototype.addValidator = function (v) {
        var validators = [];
        if (this.control.validator)
            validators.push(this.control.validator);
        validators.push(v);
        this.control.setValidators(validators);
        return this;
    };
    return FormField;
}());

var InputTextComponent = /** @class */ (function () {
    function InputTextComponent() {
    }
    InputTextComponent.prototype.clear = function () {
        this.request[this.field.id] = null;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], InputTextComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], InputTextComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], InputTextComponent.prototype, "request", void 0);
    InputTextComponent = __decorate([
        Component({
            selector: 'tw-input-text',
            template: "\n  <tw-template-field [field]='field'>\n   <div [formGroup]='form' [ngClass]='field.more?.nullable ? \"input-group\" :\"\"'>\n  <input [formControlName]='field.id' name='{{field.id}}' type='{{field.type}}' placeholder='{{field.placeholder}}' class='form-control' [(ngModel)]=\"request[field.id]\"\n    [readonly]='field.more?.readonly' [email]='field.type===\"email\"' />\n          <span class=\"input-group-btn\" *ngIf='field.more?.nullable && field.type!==\"hidden\"'>\n        <button class='btn btn-default' (click)='clear()'>&times;</button>\n      </span>\n</div>\n</tw-template-field>\n   "
        })
    ], InputTextComponent);
    return InputTextComponent;
}());

var InputFloatComponent = /** @class */ (function () {
    function InputFloatComponent() {
    }
    InputFloatComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.field.addValidator(Validators.pattern(/^[+-]?([0-9]*[.])?[0-9]+$/));
        this.sub = this.field.control.valueChanges.subscribe(function (value) {
            _this.request[_this.field.id] = parseFloat(value);
        });
    };
    InputFloatComponent.prototype.ngOnDestroy = function () {
        if (this.sub)
            this.sub.unsubscribe();
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], InputFloatComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], InputFloatComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], InputFloatComponent.prototype, "request", void 0);
    InputFloatComponent = __decorate([
        Component({
            selector: 'tw-input-float',
            template: "\n  <tw-template-field [field]='field'>\n    <div [formGroup]='form'>\n      <input placeholder='{{field.placeholder}}' [formControlName]='field.id' name='{{field.id}}' type='text' class='form-control' [readonly]='field.more?.readonly' />\n    </div>\n  </tw-template-field>"
        })
    ], InputFloatComponent);
    return InputFloatComponent;
}());

var SelectComponent = /** @class */ (function () {
    function SelectComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], SelectComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], SelectComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectComponent.prototype, "request", void 0);
    SelectComponent = __decorate([
        Component({
            selector: 'tw-select',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\">\n  <option value=\"undefined\" disabled selected>{{field.placeholder || ''}}</option>\n  <option [ngValue]=\"field.firstValue.value \" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>\n      <option [ngValue]=\"option[field.optionValue] \" *ngFor=\"let option of field.options \">\n        {{option[field.optionText]}}\n      </option>\n    </select>\n</div>\n  </tw-template-field>"
        })
    ], SelectComponent);
    return SelectComponent;
}());

var SelectTComponent = /** @class */ (function () {
    function SelectTComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], SelectTComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], SelectTComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectTComponent.prototype, "request", void 0);
    SelectTComponent = __decorate([
        Component({
            selector: 'tw-select-t',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\">\n      <option [ngValue]=\"field.firstValue.value\" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>\n      <option [ngValue]=\"option[field.optionValue]\" *ngFor=\"let option of field.options \">\n        {{option[field.optionText] | translate}}\n      </option>\n    </select>\n</div>\n  </tw-template-field>"
        })
    ], SelectTComponent);
    return SelectTComponent;
}());

var EnumselectComponent = /** @class */ (function () {
    function EnumselectComponent() {
    }
    EnumselectComponent.prototype.enumToArray = function (enumObj) {
        return Object.keys(enumObj).slice(Object.keys(enumObj).length / 2);
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], EnumselectComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], EnumselectComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], EnumselectComponent.prototype, "request", void 0);
    EnumselectComponent = __decorate([
        Component({
            selector: 'tw-enumselect',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <select [attr.multiple]='field.type === \"enumselect\"?null:true' [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\">\n      <option [ngValue]=\"field.firstValue.value\" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>\n      <option [ngValue]=\"option\" *ngFor=\"let option of enumToArray(field.options)\">\n        {{option | translate}}\n      </option>\n    </select>\n</div>\n  </tw-template-field>"
        })
    ], EnumselectComponent);
    return EnumselectComponent;
}());

var moment = moment_;
var DatetimePickerComponent = /** @class */ (function () {
    function DatetimePickerComponent() {
        this.daterange = {};
        this.minute = '';
        this.firstShow = true;
    }
    // differ: any;
    // constructor(private differs: KeyValueDiffers) {
    //   this.differ = differs.find({}).create();
    // }
    DatetimePickerComponent.prototype.selectedDate = function (value, datepicker) {
        if (value.start) {
            if (!this.field.more.dateOnly)
                this.request[this.field.id] = value.start;
            else
                this.request[this.field.id] = value.start.hours(12);
            this.datepicker.nativeElement.value = value.start.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm');
        }
        this.field.control.setValue(this.datepicker.nativeElement.value);
    };
    DatetimePickerComponent.prototype.showDaterangepicker = function (ev) {
        if (this.firstShow)
            this.selectedDate({ start: this.defaultValue });
        this.firstShow = false;
    };
    DatetimePickerComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.datepicker.nativeElement = this.datepicker.nativeElement;
        if (this.field.more) {
            if (this.field.more.defaultValue) {
                switch (this.field.more.defaultValue) {
                    case 'startOfDay':
                        this.defaultValue = moment().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue = moment().endOf('day');
                        break;
                    default:
                        this.defaultValue = this.field.more.defaultValue;
                        break;
                }
            }
            else {
                this.defaultValue = moment();
            }
        }
        else {
            this.field.more = {};
        }
        if (this.request[this.field.id]) {
            var v = moment(this.request[this.field.id]);
            this.defaultValue = v;
            this.field.control.setValue(v.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm'));
        }
        // if (this.field.more.dateOnly)
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01])$/))
        // else
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01]) (00|0[0-9]|1[0-9]|2[0-3]):([0-9]|[0-5][0-9])$/))
        this.options = {
            singleDatePicker: true,
            timePicker: !this.field.more.dateOnly,
            autoApply: true,
            timePicker24Hour: true,
            autoUpdateInput: false,
            startDate: this.defaultValue
        };
        if (this.field.more.options)
            for (var i in this.field.more.options)
                this.options[i] = this.field.more.options[i];
        this.sub = this.field.valueChange.subscribe(function (v) {
            _this.datepickerComponent.render();
            _this.datepickerComponent.selected.next({ start: moment(v) });
            _this.datepickerComponent.datePicker.setStartDate(moment(v));
            // this.selectedDate({ start: moment(v) })
        });
    };
    // valueChanged(date: Date): void {
    //   if (this.field.more.dateOnly && date)
    //     this.request[this.field.id] = date.toString()
    // }
    DatetimePickerComponent.prototype.clear = function () {
        this.request[this.field.id] = null;
        this.datepicker.nativeElement.value = '';
        this.field.control.setValue('');
    };
    DatetimePickerComponent.prototype.ngOnDestroy = function () {
        if (this.sub)
            this.sub.unsubscribe();
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], DatetimePickerComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], DatetimePickerComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DatetimePickerComponent.prototype, "request", void 0);
    __decorate([
        ViewChild('datepicker'),
        __metadata("design:type", ElementRef)
    ], DatetimePickerComponent.prototype, "datepicker", void 0);
    __decorate([
        ViewChild(DaterangePickerComponent),
        __metadata("design:type", DaterangePickerComponent)
    ], DatetimePickerComponent.prototype, "datepickerComponent", void 0);
    DatetimePickerComponent = __decorate([
        Component({
            selector: 'tw-datetime-picker',
            //   template: `
            //   <div [formGroup]='form' class="input-group">
            //       <input [formControlName]='field.id' name='{{field.id}}' class='form-control' (valueChanged)='valueChanged($event)' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false" [(ngModel)]="request[field.id]" [date-only]="field.more.dateOnly ||false" [date-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"' [parse-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"'  readonly/>
            //       <span class="input-group-btn">
            //         <button class='btn btn-default' (click)='clear()'>&times;</button>
            //       </span>
            // </div>
            //   `
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form' class=\"input-group\">\n  <input placeholder='{{field.placeholder}}' #datepicker class='form-control' type=\"text\" [formControlName]='field.id' name='{{field.id}}' id='{{field.id}}' daterangepicker [options]=\"options\" (selected)=\"selectedDate($event, daterange)\" (showDaterangepicker)=\"showDaterangepicker($event)\"  readonly/>\n  <span class=\"input-group-btn\">\n  <button class='btn btn-default' (click)='clear()'>&times;</button>\n  </span>\n  </div></tw-template-field>"
        })
    ], DatetimePickerComponent);
    return DatetimePickerComponent;
}());

var moment$1 = moment_;
var DateRangePickerComponent = /** @class */ (function () {
    function DateRangePickerComponent(differs) {
        this.differs = differs;
        this.daterange = {};
        this.minute = '';
        this.firstShow = true;
        this.differ = differs.find({}).create();
    }
    DateRangePickerComponent.prototype.selectedDate = function (value, datepicker) {
        if (value.start && value.end) {
            if (!this.field.more.dateOnly) {
                this.request[this.field.more.model1] = value.start;
                this.request[this.field.more.model2] = value.end;
            }
            else {
                this.request[this.field.more.model1] = value.start.hours(12);
                this.request[this.field.more.model2] = value.end.hours(12);
            }
            this.datepicker.nativeElement.value = value.start.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm') + ' - ' + value.end.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm');
        }
        this.field.control.setValue(this.datepicker.nativeElement.value);
    };
    DateRangePickerComponent.prototype.showDaterangepicker = function (ev) {
        if (this.firstShow)
            this.selectedDate({ start: this.defaultValue1, end: this.defaultValue2 });
        this.firstShow = false;
    };
    DateRangePickerComponent.prototype.ngOnInit = function () {
        this.datepicker.nativeElement = this.datepicker.nativeElement;
        if (this.field.more) {
            if (this.field.more.defaultValue1) {
                switch (this.field.more.defaultValue1) {
                    case 'startOfDay':
                        this.defaultValue1 = moment$1().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue1 = moment$1().endOf('day');
                        break;
                    default:
                        this.defaultValue1 = this.field.more.defaultValue1;
                        break;
                }
            }
            else {
                this.defaultValue1 = moment$1();
            }
            if (this.field.more.defaultValue2) {
                switch (this.field.more.defaultValue2) {
                    case 'startOfDay':
                        this.defaultValue2 = moment$1().startOf('day');
                        break;
                    case 'endOfDay':
                        this.defaultValue2 = moment$1().endOf('day');
                        break;
                    default:
                        this.defaultValue2 = this.field.more.defaultValue2;
                        break;
                }
            }
            else {
                this.defaultValue2 = moment$1();
            }
        }
        else {
            this.field.more = {};
        }
        if (this.request[this.field.more.model1]) {
            var v = moment$1(this.request[this.field.more.model1]);
            this.defaultValue1 = v;
        }
        if (this.request[this.field.more.model2]) {
            var v = moment$1(this.request[this.field.more.model2]);
            this.defaultValue2 = v;
        }
        if (this.request[this.field.more.model1] && this.request[this.field.more.model2])
            this.field.control.setValue(this.defaultValue1.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm') + ' - ' + this.defaultValue2.format(this.field.more.dateFormat || 'YYYY-MM-DD HH:mm'));
        // if (this.field.more.dateOnly)
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01])$/))
        // else
        // this.field.addValidator(Validators.pattern(/^\d\d\d\d-(0?[1-9]|1[0-2])-(0?[1-9]|[12][0-9]|3[01]) (00|0[0-9]|1[0-9]|2[0-3]):([0-9]|[0-5][0-9])$/))
        this.options = {
            timePicker: !this.field.more.dateOnly,
            timePicker24Hour: true,
            autoUpdateInput: false,
            autoApply: true,
            startDate: this.defaultValue1,
            endDate: this.defaultValue2
        };
        if (this.field.more.options)
            for (var i in this.field.more.options)
                this.options[i] = this.field.more.options[i];
    };
    // valueChanged(date: Date): void {
    //   if (this.field.more.dateOnly && date)
    //     this.request[this.field.id] = date.toString()
    // }
    DateRangePickerComponent.prototype.clear = function () {
        this.request[this.field.more.model1] = null;
        this.request[this.field.more.model2] = null;
        this.datepicker.nativeElement.value = '';
        this.field.control.setValue('');
    };
    DateRangePickerComponent.prototype.ngDoCheck = function () {
        var _this = this;
        var changes = this.differ.diff(this.request);
        if (changes) {
            var start_1 = null;
            var end_1 = null;
            changes.forEachChangedItem(function (r) {
                if (r.key === _this.field.more.model1) {
                    start_1 = r.currentValue;
                }
                else if (r.key === _this.field.more.model2) {
                    end_1 = r.currentValue;
                }
            });
            if (start_1 || end_1) {
                this.selectedDate({ start: start_1 || moment$1(this.request[this.field.more.model1]), end: end_1 || moment$1(this.request[this.field.more.model2]) });
                this.firstShow = false;
            }
        }
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], DateRangePickerComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], DateRangePickerComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DateRangePickerComponent.prototype, "request", void 0);
    __decorate([
        ViewChild('datepicker'),
        __metadata("design:type", ElementRef)
    ], DateRangePickerComponent.prototype, "datepicker", void 0);
    DateRangePickerComponent = __decorate([
        Component({
            selector: 'tw-daterange-picker',
            //   template: `
            //   <div [formGroup]='form' class="input-group">
            //       <input [formControlName]='field.id' name='{{field.id}}' class='form-control' (valueChanged)='valueChanged($event)' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false" [(ngModel)]="request[field.id]" [date-only]="field.more.dateOnly ||false" [date-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"' [parse-format]='this.field.more.dateFormat || "YYYY-MM-DD HH:mm"'  readonly/>
            //       <span class="input-group-btn">
            //         <button class='btn btn-default' (click)='clear()'>&times;</button>
            //       </span>
            // </div>
            //   `
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form' class=\"input-group\">\n  <input #datepicker placeholder='{{field.placeholder}}' class='form-control' type=\"text\" [formControlName]='field.id' name='{{field.id}}' daterangepicker [options]=\"options\" (selected)=\"selectedDate($event, daterange)\"  (showDaterangepicker)=\"showDaterangepicker($event)\" readonly/>\n  <span class=\"input-group-btn\">\n  <button class='btn btn-default' (click)='clear()'>&times;</button>\n  </span>\n  </div></tw-template-field>"
        }),
        __metadata("design:paramtypes", [KeyValueDiffers])
    ], DateRangePickerComponent);
    return DateRangePickerComponent;
}());

var TimePickerComponent = /** @class */ (function () {
    function TimePickerComponent() {
        this.defaultValue = '';
        this.hours = [];
        for (var i = 0; i <= 23; i++) {
            if (i < 10)
                this.hours.push('0' + i.toString());
            else
                this.hours.push(i.toString());
        }
        this.minutes = [];
        for (var i = 0; i <= 59; i++) {
            if (i < 10)
                this.minutes.push('0' + i.toString());
            else
                this.minutes.push(i.toString());
        }
    }
    // get model(): string {
    //   return this.request[this.field.id]
    // }
    // set model(m: string) {
    //   console.log(this.hour)
    // }
    TimePickerComponent.prototype.onChange = function (hour) {
        if (hour && !this.minute)
            this.minute = '00';
        if (this.hour && this.minute)
            this.request[this.field.id] = this.hour + ':' + this.minute;
        else
            this.request[this.field.id] = null;
    };
    TimePickerComponent.prototype.ngOnChanges = function (value) {
        var newV = value['request'].currentValue[this.field.id];
        if (newV) {
            var split = newV.split(':');
            this.hour = split[0] ? split[0] : '';
            this.minute = split[1] ? split[1] : '';
        }
    };
    TimePickerComponent.prototype.ngOnInit = function () {
        if (this.field.more) {
            if (this.field.more.defaultValue) ;
        }
        else {
            this.field.more = {};
        }
        // this.field.addValidator(Validators.pattern(/^(00|0[0-9]|1[0-9]|2[0-3]):(0[0-9]|[0-5][0-9])(:([0-9]|[0-5][0-9]))?$/))
    };
    // valueChanged(date: Date): void {
    //   // console.log('time changed')
    //   if (date)
    //     this.request[this.field.id] = date.toString()
    // }
    TimePickerComponent.prototype.clear = function () {
        this.hour = '';
        this.minute = '';
        this.request[this.field.id] = null;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TimePickerComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TimePickerComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TimePickerComponent.prototype, "request", void 0);
    TimePickerComponent = __decorate([
        Component({
            selector: 'tw-time-picker',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form' style='display:none'>\n  <input type='hidden'  [formControlName]='field.id' [(ngModel)]=\"this.request[this.field.id]\" />\n  </div>\n  <div class='clear' >\n   <select [(ngModel)]=\"hour\" (ngModelChange)='onChange(true)' class='form-control pull-left' style='width:100px;'>\n        <option [ngValue]=\"\"></option>\n        <option *ngFor=\"let h of hours\" [ngValue]=\"h\">{{h}}</option>\n    </select>\n    <span class='pull-left' style='margin:5px'>hh</span>\n     <select [(ngModel)]=\"minute\" (ngModelChange)='onChange()'  class='form-control pull-left' style='width:100px;'>\n        <option [ngValue]=\"\"></option>\n        <option *ngFor=\"let m of minutes\" [ngValue]=\"m\">{{m}}</option>\n    </select>\n    <span class='pull-left' style='margin:5px'>mm</span>\n      <button class='btn btn-default pull-left'  (click)='clear()'>&times;</button>\n  </div>\n    </tw-template-field>"
        })
        // <div [formGroup]='form' class="input-group">
        //   <input [formControlName]='field.id' name='{{field.id}}' class='form-control' ngui-datetime-picker default-value='{{defaultValue}}' [close-on-select]="field.more.closeOnSelect || false"  (valueChanged)='valueChanged($event)'  [(ngModel)]="request[field.id]"  [time-only]="true" date-format='HH:mm' parse-format='HH:mm'  readonly/>
        //       <span class="input-group-btn">
        //       <button class='btn btn-default' (click)='clear()'>&times;</button>
        //     </span>
        // </div>
        ,
        __metadata("design:paramtypes", [])
    ], TimePickerComponent);
    return TimePickerComponent;
}());

var SwitchComponent = /** @class */ (function () {
    function SwitchComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], SwitchComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], SwitchComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SwitchComponent.prototype, "request", void 0);
    SwitchComponent = __decorate([
        Component({
            selector: 'tw-switch',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <label class=\"switch switch-icon switch-pill switch-primary-outline-alt\">\n        <input type=\"checkbox\" [formControlName]='field.id' class=\"switch-input\"  [(ngModel)]=\"request[field.id]\" >\n        <span class=\"switch-label\" data-on=\"yes\" data-off=\"no\"></span>\n        <span class=\"switch-handle\"></span>\n    </label>\n</div>\n  </tw-template-field>"
        })
    ], SwitchComponent);
    return SwitchComponent;
}());

var GroupSelectComponent = /** @class */ (function () {
    function GroupSelectComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], GroupSelectComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], GroupSelectComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], GroupSelectComponent.prototype, "request", void 0);
    GroupSelectComponent = __decorate([
        Component({
            selector: 'tw-groupselect',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\" >\n        <option [ngValue]=\"field.firstValue.value \" *ngIf='field.firstValue' >{{field.firstValue.text | translate}}</option>\n        <optgroup label=\"{{group.label}}\" *ngFor=\"let group of field.options\">\n        <option [ngValue]=\"option[field.optionValue]\" *ngFor=\"let option of group.values\">\n          {{option[field.optionText]}}\n        </option>\n      </optgroup>\n    </select>\n</div>\n  </tw-template-field>"
        })
    ], GroupSelectComponent);
    return GroupSelectComponent;
}());

var MultiSelectComponent = /** @class */ (function () {
    function MultiSelectComponent() {
    }
    MultiSelectComponent.prototype.ngOnInit = function () {
        if (!this.field.more)
            this.field.more = {};
    };
    Object.defineProperty(MultiSelectComponent.prototype, "allSelected", {
        get: function () {
            return this.testSelected();
        },
        enumerable: true,
        configurable: true
    });
    MultiSelectComponent.prototype.selectAll = function () {
        var _this = this;
        if (this.testSelected())
            this.request[this.field.id] = [];
        else {
            this.request[this.field.id] = [];
            this.field.options.forEach(function (option) {
                _this.request[_this.field.id].push(option[_this.field.optionValue]);
            });
        }
    };
    MultiSelectComponent.prototype.testSelected = function () {
        if (!this.request[this.field.id] || !this.field.options)
            return false;
        if (this.request[this.field.id].length === this.field.options.length)
            return true;
        if (this.request[this.field.id].length === this.field.options.size)
            return true;
        return false;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], MultiSelectComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], MultiSelectComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], MultiSelectComponent.prototype, "request", void 0);
    MultiSelectComponent = __decorate([
        Component({
            selector: 'tw-multiselect',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n   <button style='margin-top: -30px; margin-bottom: 5px;' [ngClass]=\"allSelected ? 'btn-primary' : 'btn-default'\" class='btn-sm btn pull-right' (click)='selectAll()' > {{field.more.allText || 'All'}}</button>\n<select multiple [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\">\n    <option [ngValue]=\"option[field.optionValue]\" *ngFor=\"let option of field.options\">\n      {{option[field.optionText]}}\n    </option>\n  </select>\n  </div>\n  </tw-template-field>"
        })
    ], MultiSelectComponent);
    return MultiSelectComponent;
}());

var MultiSelectCBComponent = /** @class */ (function () {
    function MultiSelectCBComponent() {
    }
    MultiSelectCBComponent.prototype.ngOnInit = function () {
        if (!this.field.more)
            this.field.more = {};
    };
    Object.defineProperty(MultiSelectCBComponent.prototype, "allSelected", {
        get: function () {
            return this.testSelected();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MultiSelectCBComponent.prototype, "options", {
        get: function () {
            var _this = this;
            if (this.field.options)
                return this.field.options.map(function (opt) {
                    opt.id = opt[_this.field.optionValue];
                    opt.name = opt[_this.field.optionText];
                    return opt;
                });
            return [];
        },
        enumerable: true,
        configurable: true
    });
    MultiSelectCBComponent.prototype.selectAll = function () {
        var _this = this;
        if (this.testSelected())
            this.request[this.field.id] = [];
        else {
            this.request[this.field.id] = [];
            this.field.options.forEach(function (option) {
                _this.request[_this.field.id].push(option[_this.field.optionValue]);
            });
        }
    };
    MultiSelectCBComponent.prototype.testSelected = function () {
        if (!this.request[this.field.id] || !this.field.options)
            return false;
        if (this.request[this.field.id].length === this.field.options.length)
            return true;
        if (this.request[this.field.id].length === this.field.options.size)
            return true;
        return false;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], MultiSelectCBComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], MultiSelectCBComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], MultiSelectCBComponent.prototype, "request", void 0);
    MultiSelectCBComponent = __decorate([
        Component({
            selector: 'tw-multiselectcb',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n    <button style='margin-top: -30px; margin-bottom: 5px;' [ngClass]=\"allSelected ? 'btn-primary' : 'btn-default'\" class='btn-sm btn pull-right' (click)='selectAll()' > {{field.more.allText || 'All'}}</button>\n    <ss-multiselect-dropdown [formControlName]='field.id' [options]=\"options\" [(ngModel)]=\"request[field.id]\" ></ss-multiselect-dropdown>\n  </div>\n  </tw-template-field>"
        })
    ], MultiSelectCBComponent);
    return MultiSelectCBComponent;
}());

var TextAreaComponent = /** @class */ (function () {
    function TextAreaComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TextAreaComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TextAreaComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TextAreaComponent.prototype, "request", void 0);
    TextAreaComponent = __decorate([
        Component({
            selector: 'tw-textarea',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n   <textarea placeholder='{{field.placeholder}}' [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\"></textarea>\n</div>\n  </tw-template-field>"
        })
    ], TextAreaComponent);
    return TextAreaComponent;
}());

var DatePickerComponent = /** @class */ (function () {
    function DatePickerComponent() {
        this.defaultValue = '';
        this.closeOnSelect = false;
    }
    // private sub: Subscription
    DatePickerComponent.prototype.ngOnInit = function () {
        if (!this.field.more)
            this.field.more = {};
        this.field.more.dateOnly = true;
        if (!this.field.more.dateFormat)
            this.field.more.dateFormat = 'YYYY-MM-DD';
    };
    DatePickerComponent.prototype.ngOnDestroy = function () {
        // if (this.sub)
        //   this.sub.unsubscribe()
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], DatePickerComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], DatePickerComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DatePickerComponent.prototype, "request", void 0);
    DatePickerComponent = __decorate([
        Component({
            selector: 'tw-date-picker',
            template: "<tw-datetime-picker [form]='form' [field]='field'  [request]='request'></tw-datetime-picker>"
        })
    ], DatePickerComponent);
    return DatePickerComponent;
}());

var DaysOfWeekComponent = /** @class */ (function () {
    function DaysOfWeekComponent() {
    }
    DaysOfWeekComponent.prototype.ngOnInit = function () {
        this.field.options = DaysOfWeekValues;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], DaysOfWeekComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], DaysOfWeekComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], DaysOfWeekComponent.prototype, "request", void 0);
    DaysOfWeekComponent = __decorate([
        Component({
            selector: 'tw-dof',
            template: "<tw-enumselect [form]='form' [field]='field' [request]='request'></tw-enumselect>"
        })
    ], DaysOfWeekComponent);
    return DaysOfWeekComponent;
}());
var DaysOfWeekValues;
(function (DaysOfWeekValues) {
    DaysOfWeekValues[DaysOfWeekValues["Sunday"] = 0] = "Sunday";
    DaysOfWeekValues[DaysOfWeekValues["Monday"] = 1] = "Monday";
    DaysOfWeekValues[DaysOfWeekValues["Tuesday"] = 2] = "Tuesday";
    DaysOfWeekValues[DaysOfWeekValues["Wednesday"] = 3] = "Wednesday";
    DaysOfWeekValues[DaysOfWeekValues["Thursday"] = 4] = "Thursday";
    DaysOfWeekValues[DaysOfWeekValues["Friday"] = 5] = "Friday";
    DaysOfWeekValues[DaysOfWeekValues["Saturday"] = 6] = "Saturday";
})(DaysOfWeekValues || (DaysOfWeekValues = {}));

var TreeViewTxtComponent = /** @class */ (function () {
    function TreeViewTxtComponent() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    TreeViewTxtComponent.prototype.checkItem = function (item) {
        var e_1, _a;
        var selection = item.getSelection();
        var values = [];
        if (item.children) {
            if (selection.uncheckedItems.length === 0)
                values.push(item.value);
            try {
                for (var _b = __values(item.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    values = values.concat(this.checkItem(child));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        return values;
    };
    TreeViewTxtComponent.prototype.updateModel = function (values) {
        var e_2, _a;
        /*Select Parent*/
        if (this.treeviewComponent) {
            try {
                for (var _b = __values(this.treeviewComponent.items), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var item = _c.value;
                    values = values.concat(this.checkItem(item));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
        if (values)
            this.field.setValue(values.join(','));
        else
            this.field.setValue('');
    };
    TreeViewTxtComponent.prototype.ngOnInit = function () {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TreeViewTxtComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TreeViewTxtComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewTxtComponent.prototype, "request", void 0);
    __decorate([
        ViewChild(TreeviewComponent),
        __metadata("design:type", TreeviewComponent)
    ], TreeViewTxtComponent.prototype, "treeviewComponent", void 0);
    TreeViewTxtComponent = __decorate([
        Component({
            selector: 'tw-treeview-txt',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n   <ngx-treeview #treeview *ngIf='field.type===\"treeview-txt\"' id=\"treeview_{{field.id}}\" [config]=\"selectConfig\" [items]=\"field.options\" (selectedChange)=\"updateModel($event)\"></ngx-treeview>\n   <ngx-dropdown-treeview *ngIf='field.type===\"dropdown-treeview-txt\"' id=\"treeview_{{field.id}}\" [config]=\"selectConfig\" [items]=\"field.options\" (selectedChange)=\"updateModel($event)\"></ngx-dropdown-treeview>\n    <input [formControlName]='field.id' name='{{field.id}}' type='hidden' class='form-control' [(ngModel)]=\"request[field.id]\"/>\n</div>\n  </tw-template-field>"
        })
    ], TreeViewTxtComponent);
    return TreeViewTxtComponent;
}());

var TreeViewRadioComponent = /** @class */ (function () {
    function TreeViewRadioComponent() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    TreeViewRadioComponent.prototype.toggleCollapseExpand = function () {
        var _this = this;
        this.allItem.collapsed = !this.allItem.collapsed;
        if (this.field.options.items) {
            this.field.options.items.forEach(function (item) { return item.setCollapsedRecursive(_this.allItem.collapsed); });
        }
    };
    TreeViewRadioComponent.prototype.updateModel = function (values) {
        if (values)
            this.field.setValue(values.join(','));
        else
            this.field.setValue('');
    };
    TreeViewRadioComponent.prototype.ngOnInit = function () {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TreeViewRadioComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TreeViewRadioComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewRadioComponent.prototype, "request", void 0);
    TreeViewRadioComponent = __decorate([
        Component({
            selector: 'tw-treeview-radio',
            template: "<tw-template-field [field]='field'>\n<ng-template #tpl let-item=\"item\"\n    let-toggleCollapseExpand=\"toggleCollapseExpand\"\n    let-onCheckedChange=\"onCheckedChange\">\n    <div class=\"form-check\" [formGroup]='form'>\n        <i *ngIf=\"item.children\" (click)=\"toggleCollapseExpand()\" aria-hidden=\"true\"\n            class=\"fa\" [class.fa-caret-right]=\"item.collapsed\" [class.fa-caret-down]=\"!item.collapsed\"></i>\n        <label class=\"form-check-label\">\n            <input type=\"radio\"  class=\"form-check-input\" [(ngModel)]=\"request[field.id]\" [formControlName]='field.id'  name='{{field.id}}' [value]='item.value' />\n            {{item.text}}\n        </label>\n    </div>\n</ng-template>\n<div class=\"treeview-header\">\n\n</div>\n<div class=\"treeview-container\" [style.max-height.px]=\"selectConfig.maxHeight\" >\n    <div *ngFor=\"let item of field.options\">\n        <tw-treeview-item [item]=\"item\" [template]=\" tpl\" >\n        </tw-treeview-item>\n    </div>\n</div>\n</tw-template-field>",
            styles: ["\n.row-margin {\n    margin-top: .3rem;\n}\n.label-item-all {\n}\n.label-collapse-expand {\n    margin: 0;\n    padding: 0 .3rem;\n    cursor: pointer;\n}\n.divider {\n    height: 1px;\n    margin: 0.5rem 0;\n    overflow: hidden;\n    background: #000;\n}\n.treeview-container {\n    overflow-x: hidden;\n    overflow-y: auto;\n    padding-right: 18px;\n}\n.treeview-text {\n    padding: .3rem 0;\n    white-space: nowrap;\n}\n"]
        })
    ], TreeViewRadioComponent);
    return TreeViewRadioComponent;
}());

var TreeViewComponent = /** @class */ (function () {
    function TreeViewComponent() {
        this.selectConfig = TreeviewConfig.create({
            hasAllCheckBox: true,
            hasFilter: true,
            hasCollapseExpand: true,
            decoupleChildFromParent: false,
            maxHeight: 500
        });
    }
    TreeViewComponent.prototype.checkItem = function (item) {
        var e_1, _a;
        var selection = item.getSelection();
        var values = [];
        if (item.children) {
            if (selection.uncheckedItems.length === 0)
                values.push(item.value);
            try {
                for (var _b = __values(item.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var child = _c.value;
                    values = values.concat(this.checkItem(child));
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_1) throw e_1.error; }
            }
        }
        return values;
    };
    TreeViewComponent.prototype.updateModel = function (values) {
        var e_2, _a;
        /*Select Parent*/
        if (this.treeviewComponent) {
            try {
                for (var _b = __values(this.treeviewComponent.items), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var item = _c.value;
                    values = values.concat(this.checkItem(item));
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
        }
        if (values)
            this.field.setValue(values);
        else
            this.field.setValue([]);
    };
    TreeViewComponent.prototype.ngOnInit = function () {
        if (this.field.more && this.field.more.selectConfig)
            this.selectConfig = this.field.more.selectConfig;
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TreeViewComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TreeViewComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], TreeViewComponent.prototype, "request", void 0);
    __decorate([
        ViewChild(TreeviewComponent),
        __metadata("design:type", TreeviewComponent)
    ], TreeViewComponent.prototype, "treeviewComponent", void 0);
    TreeViewComponent = __decorate([
        Component({
            selector: 'tw-treeview',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n   <ngx-treeview *ngIf='field.type===\"treeview\"' [config]=\"selectConfig\" [items]=\"field.options\" (selectedChange)=\"updateModel($event)\"></ngx-treeview>\n   <ngx-dropdown-treeview *ngIf='field.type===\"dropdown-treeview\"'  [config]=\"selectConfig\" [items]=\"field.options\" (selectedChange)=\"updateModel($event)\"></ngx-dropdown-treeview>\n  <input [formControlName]='field.id' name='{{field.id}}' type='hidden' class='form-control' [(ngModel)]=\"request[field.id]\"/>\n</div>\n  </tw-template-field>"
        })
    ], TreeViewComponent);
    return TreeViewComponent;
}());

var SelectArrayComponent = /** @class */ (function () {
    function SelectArrayComponent() {
    }
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], SelectArrayComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], SelectArrayComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], SelectArrayComponent.prototype, "request", void 0);
    SelectArrayComponent = __decorate([
        Component({
            selector: 'tw-select-array',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <select [formControlName]='field.id' name='{{field.id}}' class='form-control' [(ngModel)]=\"request[field.id]\">\n      <option [ngValue]=\"null\" selected disabled  >{{field.firstValue | translate}}</option>\n      <option [ngValue]=\"option\" *ngFor=\"let option of field.options \">\n        {{option}}\n      </option>\n    </select>\n</div>\n  </tw-template-field>"
        })
    ], SelectArrayComponent);
    return SelectArrayComponent;
}());

var HtmlComponent = /** @class */ (function () {
    function HtmlComponent() {
        this.config = {
            height: '250px',
            toolbar: [
                ['misc', ['undo', 'redo', 'codeBlock']],
                ['font', ['bold', 'italic', 'underline', 'strikethrough', 'superscript', 'subscript', 'clear']],
                ['fontsize', ['fontname', 'fontsize', 'color']],
                ['para', ['style0', 'ul', 'ol', 'paragraph', 'height']],
                ['insert', ['table', 'picture', 'link', 'video', 'hr']],
            ]
        };
        // public onChange({ editor }: ChangeEvent) {
        //   this.request[this.field.id] = editor.getData();
        // }
    }
    HtmlComponent.prototype.ngOnInit = function () {
        if (!this.request[this.field.id])
            this.request[this.field.id] = '';
    };
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], HtmlComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], HtmlComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], HtmlComponent.prototype, "request", void 0);
    HtmlComponent = __decorate([
        Component({
            selector: 'tw-html',
            template: "<tw-template-field [field]='field'>\n  <div [formGroup]='form'>\n  <div [ngxSummernote]=\"field?.more?.config || config\" [ngxSummernoteView]=\"request[field.id]\"  [formControlName]='field.id' [(ngModel)]=\"request[field.id]\"></div>\n  </div>\n  </tw-template-field>"
        })
    ], HtmlComponent);
    return HtmlComponent;
}());

var ReactiveFormComponent = /** @class */ (function () {
    function ReactiveFormComponent(componentFactoryResolver, renderer) {
        this.componentFactoryResolver = componentFactoryResolver;
        this.renderer = renderer;
        this.fields = [];
        this.events = []; // use later to display form changes
        this.afterInit = new EventEmitter(true);
        this.afterViewInit = new EventEmitter(true);
    }
    ReactiveFormComponent.prototype.ngOnChanges = function (values) {
        if (values.fields && !values.fields.firstChange)
            this.refreshComponentsFields();
        if (values.request && !values.request.firstChange)
            this.components.forEach(function (c) { return c.request = values.request.currentValue; });
        if (values.form && !values.form.firstChange)
            this.components.forEach(function (c) { return c.form = values.form.currentValue; });
    };
    ReactiveFormComponent.prototype.ngOnInit = function () {
        this.refreshComponentsFields();
        this.afterInit.next(true);
    };
    ReactiveFormComponent.prototype.ngAfterViewInit = function () {
        this.afterViewInit.next(true);
    };
    ReactiveFormComponent.prototype.refreshComponentsFields = function () {
        var _this = this;
        this.vc.clear();
        this.components = [];
        this.fields.forEach(function (field) {
            _this.form.addControl(field.id, field.control);
            _this.components.push(_this.loadComponentFields(field));
        });
    };
    ReactiveFormComponent.prototype.loadComponentFields = function (f) {
        var componentFactory = this.componentFactoryResolver.resolveComponentFactory(this.getComponent(f));
        var componentRef = this.vc.createComponent(componentFactory);
        componentRef.instance.field = f;
        componentRef.instance.form = this.form;
        componentRef.instance.request = this.request;
        this.renderer.addClass(componentRef.location.nativeElement, 'form-group');
        this.renderer.addClass(componentRef.location.nativeElement, 'col-' + (f.col || 'sm-6'));
        if (f.class)
            this.renderer.addClass(componentRef.location.nativeElement, f.class);
        return componentRef.instance;
    };
    ReactiveFormComponent.prototype.getComponent = function (f) {
        var comp = null;
        switch (f.type) {
            case 'float':
                comp = InputFloatComponent;
                break;
            case 'select':
                comp = SelectComponent;
                break;
            case 'selectT':
                comp = SelectTComponent;
                break;
            case 'arrayselect':
                comp = SelectArrayComponent;
                break;
            case 'enumselect':
                comp = EnumselectComponent;
                break;
            case 'datetime':
                comp = DatetimePickerComponent;
                break;
            case 'daterange':
                comp = DateRangePickerComponent;
                break;
            case 'time':
                comp = TimePickerComponent;
                break;
            case 'boolean':
                comp = SwitchComponent;
                break;
            case 'groupselect':
                comp = GroupSelectComponent;
                break;
            case 'multiselect':
                comp = MultiSelectComponent;
                break;
            case 'multiselectcb':
                comp = MultiSelectCBComponent;
                break;
            case 'textarea':
                comp = TextAreaComponent;
                break;
            case 'date':
                comp = DatePickerComponent;
                break;
            case 'daysOfWeek':
                comp = DaysOfWeekComponent;
                break;
            case 'treeview-txt':
                comp = TreeViewTxtComponent;
                break;
            case 'treeview-radio':
                comp = TreeViewRadioComponent;
                break;
            case 'treeview':
                comp = TreeViewComponent;
                break;
            case 'html':
                comp = HtmlComponent;
                break;
            case 'text':
            default:
                comp = InputTextComponent;
                break;
        }
        return comp;
    };
    __decorate([
        Input(),
        __metadata("design:type", Array)
    ], ReactiveFormComponent.prototype, "fields", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], ReactiveFormComponent.prototype, "form", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Object)
    ], ReactiveFormComponent.prototype, "request", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], ReactiveFormComponent.prototype, "afterInit", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], ReactiveFormComponent.prototype, "afterViewInit", void 0);
    __decorate([
        ViewChild('vc', { read: ViewContainerRef }),
        __metadata("design:type", Object)
    ], ReactiveFormComponent.prototype, "vc", void 0);
    ReactiveFormComponent = __decorate([
        Component({
            selector: 'tw-reactive-form',
            template: "\n  <form [formGroup]=\"form\" >\n  <div class=\"row\" >\n  <ng-container #vc></ng-container>\n  </div>\n  </form>\n    "
        }),
        __metadata("design:paramtypes", [ComponentFactoryResolver, Renderer2])
    ], ReactiveFormComponent);
    return ReactiveFormComponent;
}());

// import { TreeviewItemTemplateContext } from 'ng2-dropdown-treeview/src/treeview-item-template-context';
var TreeViewItemComponent = /** @class */ (function () {
    function TreeViewItemComponent() {
        var _this = this;
        this.toggleCollapseExpand = function () {
            _this.item.collapsed = !_this.item.collapsed;
        };
    }
    __decorate([
        Input(),
        __metadata("design:type", TemplateRef)
    ], TreeViewItemComponent.prototype, "template", void 0);
    __decorate([
        Input(),
        __metadata("design:type", TreeviewItem)
    ], TreeViewItemComponent.prototype, "item", void 0);
    TreeViewItemComponent = __decorate([
        Component({
            selector: 'tw-treeview-item',
            template: "\n<div class=\"treeview-item\">\n    <ng-template [ngTemplateOutlet]=\"template\"\n        [ngTemplateOutletContext]=\"{item: item, toggleCollapseExpand: toggleCollapseExpand}\">\n    </ng-template>\n    <div *ngIf=\"!item.collapsed\">\n        <tw-treeview-item *ngFor=\"let child of item.children\" [item]=\"child\" [template]=\"template\">\n        </tw-treeview-item>\n    </div>\n</div>\n    ",
            styles: ["\n:host {\n    display: block;\n}\n:host /deep/ .fa {\n    margin-right: .2rem;\n    width: .5rem;\n    cursor: pointer;\n}\n.treeview-item {\n    white-space: nowrap;\n}\n.treeview-item .treeview-item {\n    margin-left: 2rem;\n}\n    "]
        })
    ], TreeViewItemComponent);
    return TreeViewItemComponent;
}());

var BusyService = /** @class */ (function () {
    function BusyService() {
        this.global = false;
    }
    BusyService.prototype.start = function () {
        this.global = true;
    };
    BusyService.prototype.stop = function () {
        this.global = false;
    };
    BusyService = __decorate([
        Injectable()
    ], BusyService);
    return BusyService;
}());

var BusyButtonComponent = /** @class */ (function () {
    function BusyButtonComponent(busy) {
        this.busy = busy;
        this.onClick = new EventEmitter(true);
    }
    BusyButtonComponent.prototype.send = function () {
        this.onClick.next(true);
    };
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], BusyButtonComponent.prototype, "cssClass", void 0);
    __decorate([
        Input(),
        __metadata("design:type", Boolean)
    ], BusyButtonComponent.prototype, "disabled", void 0);
    __decorate([
        Input(),
        __metadata("design:type", String)
    ], BusyButtonComponent.prototype, "icon", void 0);
    __decorate([
        Output(),
        __metadata("design:type", EventEmitter)
    ], BusyButtonComponent.prototype, "onClick", void 0);
    BusyButtonComponent = __decorate([
        Component({
            selector: 'tw-busy-button',
            template: '<button (click)=\'send()\' [class]="cssClass" [disabled]="disabled || busy.global"><span style=\'margin- right:5px\'  *ngIf=\'busy.global\'><img src="assets/img/ajax-loader.gif"/></span><i class=\'fa fa-{{icon }}\' style=\'margin- right:5px\'  *ngIf=\'icon && !busy.global\'></i><ng-content></ng-content></button>'
        }),
        __metadata("design:paramtypes", [BusyService])
    ], BusyButtonComponent);
    return BusyButtonComponent;
}());

var AngularFormsUtilsConfigs = /** @class */ (function () {
    function AngularFormsUtilsConfigs(daterangepickerConfig) {
        this.daterangepickerConfig = daterangepickerConfig;
    }
    AngularFormsUtilsConfigs = __decorate([
        Injectable(),
        __metadata("design:paramtypes", [DaterangepickerConfig])
    ], AngularFormsUtilsConfigs);
    return AngularFormsUtilsConfigs;
}());

var TemplateFieldComponent = /** @class */ (function () {
    function TemplateFieldComponent() {
    }
    Object.defineProperty(TemplateFieldComponent.prototype, "errors", {
        get: function () {
            var errors = [];
            //
            if (this.field.control.errors != null) {
                Object.keys(this.field.control.errors).forEach(function (keyError) {
                    errors.push("form.errors." + keyError);
                });
            }
            return errors;
        },
        enumerable: true,
        configurable: true
    });
    __decorate([
        Input(),
        __metadata("design:type", FormField)
    ], TemplateFieldComponent.prototype, "field", void 0);
    __decorate([
        Input(),
        __metadata("design:type", FormGroup)
    ], TemplateFieldComponent.prototype, "form", void 0);
    TemplateFieldComponent = __decorate([
        Component({
            selector: 'tw-template-field',
            template: "\n  <label [ngClass]='{\"text-danger form-errors\":(field.control.invalid && (field.control.dirty || field.control.touched))}' for=\"{{field.id}}\"><span *ngIf='field.isRequired'>* </span>{{ field.label | translate }} <small *ngIf='field.info' > {{ field.info | translate }}</small></label>\n    <ng-content ></ng-content>\n     <ul *ngIf=\"field.control.invalid && (field.control.dirty || field.control.touched)\" class=\"text-danger form-errors\">\n        <li *ngFor=\"let error of errors\">{{error | translate}}</li>\n    </ul>\n      "
        })
    ], TemplateFieldComponent);
    return TemplateFieldComponent;
}());

// NguiDatetime.parseDate = (str: any): any => {
//   let m: any = moment(str)
//   if (!m.isValid()) { //Time
//     // let tmp = str.split(':');
//     m = moment()
//     // m.set('hour', parseInt(tmp[0] || '0', 10))
//     // m.set('minute', parseInt(tmp[1] || '0', 10))
//     // m.set('second', parseInt(tmp[2] || '0', 10))
//   }
//   return m.toDate()
// }
var decExp = [
    TemplateFieldComponent,
    ReactiveFormComponent,
    DatetimePickerComponent,
    MultiSelectCBComponent,
    SelectComponent,
    EnumselectComponent,
    InputTextComponent,
    InputFloatComponent,
    TimePickerComponent,
    SwitchComponent,
    GroupSelectComponent,
    DatePickerComponent,
    DaysOfWeekComponent,
    MultiSelectComponent,
    TextAreaComponent,
    BusyButtonComponent,
    TreeViewComponent,
    TreeViewTxtComponent,
    TreeViewRadioComponent,
    TreeViewItemComponent,
    SelectTComponent,
    DateRangePickerComponent,
    SelectArrayComponent,
    HtmlComponent
];
var AngularFormsUtilsModule = /** @class */ (function () {
    function AngularFormsUtilsModule() {
    }
    AngularFormsUtilsModule_1 = AngularFormsUtilsModule;
    AngularFormsUtilsModule.forRoot = function () {
        return { ngModule: AngularFormsUtilsModule_1, providers: [BusyService] };
    };
    var AngularFormsUtilsModule_1;
    AngularFormsUtilsModule = AngularFormsUtilsModule_1 = __decorate([
        NgModule({
            imports: [
                ReactiveFormsModule,
                CommonModule,
                TranslateModule,
                MultiselectDropdownModule,
                // NguiDatetimePickerModule,
                FormsModule,
                TreeviewModule,
                Daterangepicker,
                // FroalaEditorModule.forRoot(),
                // FroalaViewModule.forRoot(),
                NgxSummernoteModule,
                HttpClientModule
            ],
            declarations: decExp,
            exports: decExp,
            entryComponents: decExp,
            providers: [AngularFormsUtilsConfigs]
        })
    ], AngularFormsUtilsModule);
    return AngularFormsUtilsModule;
}());

/**
 * Generated bundle index. Do not edit.
 */

export { AngularFormsUtilsConfigs, AngularFormsUtilsModule, BusyService, FormField, TemplateFieldComponent as ɵa, ReactiveFormComponent as ɵb, DatetimePickerComponent as ɵc, MultiSelectCBComponent as ɵd, SelectComponent as ɵe, EnumselectComponent as ɵf, InputTextComponent as ɵg, InputFloatComponent as ɵh, TimePickerComponent as ɵi, SwitchComponent as ɵj, GroupSelectComponent as ɵk, DatePickerComponent as ɵl, DaysOfWeekComponent as ɵm, MultiSelectComponent as ɵn, TextAreaComponent as ɵo, BusyButtonComponent as ɵp, TreeViewComponent as ɵq, TreeViewTxtComponent as ɵr, TreeViewRadioComponent as ɵs, TreeViewItemComponent as ɵt, SelectTComponent as ɵu, DateRangePickerComponent as ɵv, SelectArrayComponent as ɵw, HtmlComponent as ɵx };
//# sourceMappingURL=angular-forms-utils.js.map
